package intro.decon;
import jam.vector.*;
import juice.solver.*;
import juice.operator.*;
import rsf.vector.*;
import conv.convi.*; 

/**
* I solve b = M x for x.                                             <BR>
* b is a    known time series                              (trace).  <BR>
* M is a    known convolution operator of the reflectivity (rflct).  <BR>
* x is an unknown wavelet                                  (wavlt).  <BR>
* 
* I create the normal equations: b_n = M'b and M_n = M'M and 
* call a CGSolver to solve b_n = M_n x, respectively M'b = M'M x. 
*
* <BR><!--#include file="./source.inc"--><BR> 
*/ 
public class IcafDecon { 
  public static void main(String[] args) {
    Rsf trace = RsfFactory.newRsf("./trace.H");          // reads   b
    Rsf rflct = RsfFactory.newRsf("./rflct.H");          // reads   m 
    Icaf conv = new IcafFactory(rflct).                   // creates M 
      getIcafFromOutputSpace((RsfSpace) trace.getSpace()); 
    Vector     traceNormal = conv.adjoint().image(trace); // creates b_n = M'b 
    CompoundLinOpAdj                                      // creates M_n = M'M
      convNormal = new CompoundLinOpAdj(conv, conv.adjoint());
    Rsf wavlt = (Rsf) convNormal.getDomain().newMember(); // creates x 
    new CGSolver().solve(convNormal,traceNormal,wavlt);   // solves b_n = M_n x
    wavlt.write( System.out );                            // writes x 
  }
}
