package isf.vector;

import java.util.Vector;
import  jam.JamObject;

/**
 * coordinate representation of <TT>Isf</TT>s 
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * This implements a dynamic 2-D lookup-table. 
 * Imagine you need to write down the the location of 12 cities. 
 * In an implementation of the Coordinates interface you create a 
 * table with 2 keys (longitude and latitude) and 12 element entries 
 * (the number of your cities). You then enter the position of each 
 * city in the 24 fields you created.<BR> 
 * 
 * This lookup table allows any number of keys and elements and you 
 * can add or remove elements or keys as needed.<BR> 
 *
 * Currently, I envision float and integer key values, but I do not 
 * restrict it to float and integers. I could easily by introducing 
 * an coordinate object that contains the coordinates for one key or 
 * for one element.<BR>
 *
 * @see       Isf
 * @author    Matthias Schwab
 */
interface CoordMap extends JamObject {

  JamObject copy();

  // accessors 
  CoordMap getCoordinates    ();
  int     getNumberOfElements(); 
  int     getNumberOfKeys    ();

  void rand(); 

  void    setCoordAt(float coord, int key, int element);
  void    setCoordAt(int   coord, int key, int element);
  float floatCoordAt(             int key, int element); 
  int     intCoordAt(             int key, int element); 
  Object     coordAt(             int key, int element); 

  /**
   * adds for each element in the table a set of coordinates. 
   * The new coordinates are appended to the table. 
   * (increases the number of keys by 1).
   */ 
  void       addKey      (Vector  vec             ); 
  void       addKey      (int[]   vec             );
  void       addKey      (float[] vec             );

  void    insertKeyAt    (Vector  vec, int key); 
  void    insertKeyAt    (float[] vec, int key); 
  void    insertKeyAt    (int  [] vec, int key); 
  void       setKeyAt    (Vector  vec, int key);
  void       setKeyAt    (float[] vec, int key);
  void       setKeyAt    (int  [] vec, int key);
  void    removeKeyAt    (             int key); 
  float[]  floatKeyAt    (             int key); 
  int  []    intKeyAt    (             int key); 
  Vector        keyAt    (             int key); 

  /**
   * adds for each key in the table a set of coordinates. 
   * The new coordinates are appended to the table. 
   * (increases the number of elements by 1).
   */ 
  void       addElement  (Vector vec              );

  void    insertElementAt(Vector vec,  int element); 
  void       setElementAt(Vector vec,  int element);
  void    removeElementAt(             int element); 
  Vector        elementAt(             int element);

}


