package isf.vector;

import  java.util.Vector;
import  jam.JamObject;

/**
 * coordinate representation for <TT>IsfSpace</TT>s 
 * using a Vector of Vectors <TT>Isf</TT>s
 * (fundamentally just a dynamic 2-D array)
 * 
 * maybe the fast axis should loop over elements 
 * (currently it loops over keys)
 *
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @see       IsfSpace
 * @author    Matthias Schwab
 */
public class CoordMapV implements CoordMap {
  Vector coords; // vector of vectors to define a 2-D field 

  // constructors
  public CoordMapV() { }

  public CoordMapV(int keys, int elements) { 
    coords = new Vector(elements); 
    coords.setSize(elements); 
    for( int i = 0; i < elements; i++ ) { 
      Vector tempor = new Vector(keys); 
      tempor.setSize(keys); 
      coords.setElementAt(tempor,         i);
    }
  } 
  /** copy constructor */
  protected CoordMapV(CoordMapV src) { 
    int elements = src.getNumberOfElements(); 
    int keys     = src.getNumberOfKeys    ();
    coords       = new Vector(elements); 
    coords.setSize(elements); 
    for (int j = 0; j < elements; j++) { 
      Vector tempor = new Vector(keys); 
      tempor.setSize(keys); 
      coords.setElementAt(tempor, j);
      for (int k = 0; k < keys  ; k++) 
	this.setCoordAt( src.coordAt(k,j), k,j ); 
    }
  }

  public JamObject copy() { return new CoordMapV(this); }

  public void rand() {
    java.util.Random rand = new java.util.Random    ();
    int  elements         = this.getNumberOfElements();
    int  keys             = this.getNumberOfKeys    ();
    for (int i=0; i < elements; i++ ) {
    for (int j=0; j < keys    ; j++ ) {
      this.setCoordAt(rand.nextFloat(), j, i);
    }}
  }

  // comparitors
  public boolean equals(Object spc2 ) {
    if (   this == spc2                                     ) return true;
    if ( ! (spc2 instanceof CoordMapV)                       ) return false;
    if ( ! equalCoords(   ((CoordMapV)spc2).coords          )) return false; 
    return true;
  }

  protected boolean equalCoords(Vector coord2) { 
    int  elements1  = this.coords.size();
    int  elements2  =      coord2.size();
    if ( elements1 != elements2 ) return false; 
    for (int i = 0; i < elements1; i++) 
      if ( ! juice.util.JuiceUtil.
	   equalVectors((Vector) coords.elementAt(i),
		        (Vector) coord2.elementAt(i)) ) return false;
    return true;
  }

  // accessors 

  //stats 
  public CoordMap getCoordinates     () { return (CoordMap) coords; }
  public int      getNumberOfElements() { return coords.size(); }
  public int      getNumberOfKeys    () { 
    return (coords.size() == 0) ? 0 : ((Vector) coords.elementAt(0)).size();
  }

  public void    setCoordAt(float  loc, int key, int element) {
    ((Vector) coords.elementAt(element)).setElementAt(new Float  (loc), key);
  }
  public void    setCoordAt(int    loc, int key, int element) {
    ((Vector) coords.elementAt(element)).setElementAt(new Integer(loc), key);
  }
  public void    setCoordAt(Object loc, int key, int element) {
    ((Vector) coords.elementAt(element)).setElementAt(loc, key);
  }
  public float floatCoordAt(            int key, int element) {
    Float  tmp = (Float) ((Vector) coords.elementAt(element)).elementAt(key);
    return tmp.floatValue();
  }
  public int     intCoordAt(            int key, int element) {
    Integer tmp = (Integer) ((Vector)coords.elementAt(element)).elementAt(key);
    return tmp.intValue();
  }
  public Object      coordAt(        int key, int element) {
    return ((Vector) this.coords.elementAt(element)).elementAt(key);
  }

  private Vector cloneKeys(float[] keys) {
    if (keys.length != this.getNumberOfKeys()) 
      System.err.println("*** CoordMapV: mismatch of key number");
    Vector tmp = new Vector();
    for (int i = 0; i < keys.length; i++) 
      tmp.addElement(new Float  (keys[i])); 
    return tmp;
  }   
  private Vector cloneKeys(int  [] keys) {
    if (keys.length != this.getNumberOfKeys()) 
      System.err.println("*** CoordMapV: mismatch of key number");
    Vector tmp = new Vector();
    for (int i = 0; i < keys.length; i++) 
      tmp.addElement(new Integer(keys[i])); 
    return tmp;
  }   

  // acessing a key for all elements

  public void addKey    (float[] coords) { 
    if (coords.length != this.getNumberOfElements()) 
      System.err.println("*** CoordMapV: mismatch in addNewKey");
    for (int i = 0; i < coords.length; i++) 
      ((Vector)this.coords.elementAt(i)).addElement(new Float(coords[i]));
  }
  public void addKey    (int  [] coords) { 
    if (coords.length != this.getNumberOfElements()) 
      System.err.println("*** CoordMapV: mismatch in addNewKey");
    for (int i = 0; i < coords.length; i++) 
      ((Vector)this.coords.elementAt(i)).addElement(new Integer(coords[i]));
  }
  public void addKey    (Vector  coords) { 
    if (coords.size() != this.getNumberOfElements()) 
      System.err.println("*** CoordMapV: mismatch in addNewKey");
    for (int i = 0; i < coords.size(); i++) 
      ((Vector)this.coords.elementAt(i)).addElement(coords.elementAt(i));
  }

  public void insertKeyAt(float[] coords, int key) { 
    if (coords.length != this.getNumberOfElements()) 
      System.err.println("*** CoordMapV: mismatch in insertKeyAt()");
    for (int i = 0; i < coords.length; i++) 
      ((Vector)this.coords.elementAt(i)).
	insertElementAt(new Float(coords[i]), key);
  }
  public void insertKeyAt(int  [] coords, int key) { 
    if (coords.length != this.getNumberOfElements()) 
      System.err.println("*** CoordMapV: mismatch in insertKeyAt()");
    for (int i = 0; i < coords.length; i++) 
      ((Vector)this.coords.elementAt(i)).
	insertElementAt(new Integer(coords[i]), key);
  }
  public void insertKeyAt(Vector coords, int key) { 
    if (coords.size() != this.getNumberOfElements()) 
      System.err.println("*** CoordMapV: mismatch in insertKeyAt()");
    for (int i = 0; i < coords.size(); i++) 
      ((Vector)this.coords.elementAt(i)).
	insertElementAt(coords.elementAt(i), key);
  }

  public void setKeyAt    (float[] coords, int key) {
    if (coords.length != this.getNumberOfElements()) 
      System.err.println("*** CoordMapV: mismatch in setKeyAt()");
    for (int i = 0; i < coords.length; i++) 
      ((Vector)this.coords.elementAt(i)).
	setElementAt(new Float(coords[i]), key);
  }
  public void setKeyAt    (int  [] coords, int key) {
    if (coords.length != this.getNumberOfElements()) 
      System.err.println("*** CoordMapV: mismatch in setKeyAt()");
    for (int i = 0; i < coords.length; i++) 
      ((Vector)this.coords.elementAt(i)).
	setElementAt(new Integer(coords[i]), key);
  }
  public void setKeyAt    (Vector coords, int key) {
    if (coords.size() != this.getNumberOfElements()) 
      System.err.println("*** CoordMapV: mismatch in setKeyAt()");
    for (int i = 0; i < coords.size(); i++) 
      ((Vector)this.coords.elementAt(i)).
	setElementAt(coords.elementAt(i), key);
  }

  public void removeKeyAt  (int key) { 
    for (int i = 0; i < this.getNumberOfElements(); i++) 
      ((Vector)this.coords.elementAt(i)).removeElementAt(key);
  }

  public float[] floatKeyAt(int key) {
    int n = this.getNumberOfElements();
    float[] elem = new float[n];
    for (int i=0; i < n; i++) 
      elem[i] = ((Float)((Vector)this.coords.elementAt(i)).
		 elementAt(key)).floatValue();
    return elem;
  }
  public int  []   intKeyAt(int key) {
    int n = this.getNumberOfElements();
    int[] elem = new int[n];
    for (int i=0; i < n; i++) 
      elem[i] = ((Integer)((Vector)this.coords.elementAt(i)).
		 elementAt(key)).intValue();
    return elem;
  }
  public Vector       keyAt(int key) {
    int n = this.getNumberOfElements();
    Vector elem = new Vector(n);
    for (int i=0; i < n; i++) 
      elem.addElement(((Vector)this.coords.elementAt(i)).elementAt(key));
    return elem;
  }

  // acessing all keys of an element 

  public void       addElement  (Vector coords             ) { 
        this.coords.addElement  (       coords             );
  }
  public void    insertElementAt(Vector coords, int element) { 
     this.coords.insertElementAt(       coords,     element);
  }
  public void       setElementAt(Vector coords, int element) {
        this.coords.setElementAt(       coords,     element);
  }
  public void    removeElementAt(               int element) { 
     this.coords.removeElementAt(                   element);
  }
  public Vector        elementAt(               int element) {
    return (Vector) this.coords.elementAt(          element);
  }

  // I/O

  public String toString() {
    StringBuffer buf = new StringBuffer        ();
    int  elements    = this.getNumberOfElements();
    int  keys        = this.getNumberOfKeys    ();
    for (int i=0; i < elements; i++ ) { 
      buf.append("\n element["     + i                   + "]"  ); 
      for (int j=0; j < keys  ; j++ ) {
	try {
	  buf.append("(" + j + "," + this.coordAt(j,i) + ") ");
	}
	catch(NullPointerException e) {
	  buf.append("(" + j + "," + "null"              + ") ");
	}
      }
    }
    return buf.toString();
  }
}


