package isf.vector;

import  jam.vector.*;
import  jam.JamObject;

/**
 * Set of java Vectors for <TT>Isf</TT>
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * This requires all desirable data base methods and all jam vector methods
 * for a set of jam Vectors.
 *
 * @see       Isf
 * @author    Matthias Schwab
 */
interface Elements extends JamObject { 

  // accessor 
  int getNumberOfElements();

  // java vector methods (database) - more to come .. 
  int capacity();
  
  void    insertElementAt(Vector vec, int index);
  void    removeElementAt(            int index); 
  void       addElement  (Vector vec           );
  void       setElementAt(Vector vec, int index);
  Vector        elementAt(            int index); 

  // jam vector methods

  /**
   *  this = -this
   */
  public void         neg  ();
  /**
   *  this = -vec2
   */
  public void         neg  (Elements vec2);
  /**
   *  this = this  + vec2
   */
  public void         add  (Elements vec2);
  /**
   *  this = vec1 + vec2
   */
  public void         add  (Elements vec1,
			    Elements vec2);

  /**
   *  this = this + a*vec2
   */
  public void         addScale(float a,
			       Elements vec2);
  /**
   *  this = vec1 + a*vec2
   */
  public void         addScale(float a,
			       Elements vec1,
			       Elements vec2);

  /**
   *  this = this  - vec2
   */
  public void         sub  (Elements vec2);
  /**
   *  this = vec1 - vec2
   */
  public void         sub  (Elements vec1,
			    Elements vec2);

  /**
   *  this = a*this
   */
  public void         scale(float        a    );
  /**
   *  this = a*vec
   */
  public void         scale(float        a    ,
			    Elements vec2 );

  /**
   * this = src (without reallocating this)
   */
  public void         copyFrom(Elements src);

  /**
   * this = 0 (the zero vector)
   */
  public void         zero ();

  /**
   *  @return < this, vec2 > (the inner product)
   */
  public float dot  (Elements vec2);
  /**
   *  @return < this, this > ( = norm^2(this) )
   */
  public float norm2();
  /**
   *  @return sqrt( < this, this > ) ( = norm(this) )
   */
  public float norm ();
}
