package isf.vector;

import jam.JamObject;
import jam.vector.*;

/**
 * vector space containing <TT>Isf</TT>s
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * I mix here java and jam Vectors. I cannot avoid it, so pay attention!
 * I choose to default Vector to jam Vector. The java Vector is invoked 
 * by its full length java.util.Vector.
 *
 * @see       Isf
 * @author    Matthias Schwab
 */
public class ElementsV implements Elements { 
  java.util.Vector vec; 

  //constructors 
  public ElementsV(Space spc, int dim) {
    vec = new java.util.Vector(dim);   
    for( int i = 0; i < dim; i++ ) { 
      vec.addElement(spc.newMember()); 
    }
  }
  public ElementsV(Space[]  spc) {
    int n = spc.length;
    vec = new java.util.Vector(n);
    for ( int i = 0; i < n; i++) 
      vec.addElement(spc[i].newMember()); 
  }
  public ElementsV(Spaces spc) {
    int n = spc.getNumberOfElements();
System.out.println("here is n: " + n);
    vec = new java.util.Vector(n);
    for ( int i = 0; i < n; i++) 
      vec.addElement(((Space) spc.getSpace(i)).newMember()); 
  }
  public ElementsV(Vector[] vec) {
    int n = vec.length;
    this.vec = new java.util.Vector(n);
    for( int i = 0; i < n; i++) 
      this.vec.addElement(vec[i]);
  }
  // maybe I should check that the vector elements are of type Vector.
  public ElementsV(java.util.Vector vec) { 
    /*
    if ( ! (vec.elementAt(0) instanceOf Vector) ) {
      System.err.println("***ElementsV: data isn't a java Vector of jam Vectors");
    }
    */
    this.vec = vec;
  }
  // copy constructor
  public ElementsV(ElementsV src) {
    int n = src.getNumberOfElements();
    vec = new java.util.Vector(n);
    for (int k = 0; k < n; k++) {
      vec.setElementAt(((Vector)src.vec.elementAt(k)).copy(), k);
    }
  }
  public JamObject copy() { return new ElementsV(this); }

  //comparitor
  public boolean equals(Object data ) {
    if (  this == data                             ) return true ;
    if (! (data instanceof ElementsV)                ) return false;
    if (! juice.util.JuiceUtil.equalVectors(this.vec, 
			       ((ElementsV)data).vec)) return false;
    return true; 
  }

  //accessor 
  public int getNumberOfElements() { return vec.size    (); }
  public int capacity           () { return vec.capacity(); }

  // vector - database accessor 
  public void     insertElementAt(Vector newVector, int index) {
              vec.insertElementAt(       newVector,     index);
  }
  public void     removeElementAt(                  int index) {
              vec.removeElementAt(                      index);
  }
  public void        addElement  (Vector newVector           ) {
                 vec.addElement  (       newVector           );
  }
  public void        setElementAt(Vector newVector, int index) {
                 vec.setElementAt(       newVector,     index);
  }
  public Vector         elementAt(                  int index) {
    return (Vector) vec.elementAt(                      index);
  }  

  // I/O
  public String toString() { 
    return juice.util.JuiceUtil.vectorToString("Vector",vec,"\n");
  }

  // methods from Vector
  public void neg() {
    for ( int i = 0; i < vec.size(); i++ )
      ((Vector) vec.elementAt(i)).neg();
  }
  public void neg(Elements vec2) {
    for ( int i = 0; i < vec.size(); i++ ) 
      ((Vector) vec.elementAt(i)).neg(
                      (Vector)((ElementsV) vec2).vec.elementAt(i));
  }
  public void addScale(float  a, 
		       Elements vec2) { 
    for (int i = 0; i < vec.size(); i++ ) { 
      ((Vector) vec.elementAt(i)).addScale(a,
	              (Vector) ((ElementsV) vec2).vec.elementAt(i));
    }
  }
  public void addScale(float  a,
		       Elements vec1,
		       Elements vec2) {
    for (int i = 0; i < vec.size(); i++ ) {
      ((Vector) vec.elementAt(i)).addScale(a, 
		      (Vector)((ElementsV) vec1).vec.elementAt(i),  
		      (Vector)((ElementsV) vec2).vec.elementAt(i));
    }
  }
  public void add(Elements vec2) {
    for (int i = 0; i < vec.size(); i++ )
      ((Vector) vec.elementAt(i)).add(
		      (Vector)((ElementsV) vec2).vec.elementAt(i));
  }
  public void add(Elements vec1,
		  Elements vec2) {
    for (int i = 0; i < vec.size(); i++ )
      ((Vector) vec.elementAt(i)).add(
                      (Vector)((ElementsV) vec1).vec.elementAt(i),
		      (Vector)((ElementsV) vec2).vec.elementAt(i));
  }
  public void sub(Elements vec2) {
    for (int i = 0; i < vec.size(); i++ )
      ((Vector) vec.elementAt(i)).sub(
                      (Vector)((ElementsV) vec2).vec.elementAt(i));
  }
  public void sub(Elements vec1,
		  Elements vec2) {
    for (int i = 0; i < vec.size(); i++ )
      ((Vector) vec.elementAt(i)).sub(
                      (Vector)((ElementsV) vec1).vec.elementAt(i),
		      (Vector)((ElementsV) vec2).vec.elementAt(i));
  }
  public void scale(float a) {
    for (int i = 0; i < vec.size(); i++ )
      ((Vector) this.vec.elementAt(i)).scale(a);
  }
  public void scale(float  a, Elements vec2) {
    for (int i = 0; i < vec.size(); i++ )
      ((Vector) vec.elementAt(i)).scale(a,
                      (Vector)((ElementsV) vec2).vec.elementAt(i));
  }

  // Fill
  public void copyFrom(Elements src) {
    for (int i = 0; i < vec.size(); i++ )
      ((Vector) vec.elementAt(i)).copyFrom(
                      (Vector)((ElementsV) src).vec.elementAt(i));
  }
  public void zero() { 
    for (int i = 0; i < vec.size(); i++ )
      ((Vector) vec.elementAt(i)).zero();
  }

  // Inner product and related operations
  public float dot(Elements vec2) {
    float innr = 0f;
    for (int i = 0; i < vec.size(); i++ ) {
      innr += ((Vector) vec.elementAt(i)).dot(
		      (Vector)((ElementsV) vec2).vec.elementAt(i));
    }
    return innr;
  }
  
  public float norm2() {
    float norm2 = 0;
    for (int i = 0; i <  vec.size(); i++ ) {
      norm2 += ((Vector) vec.elementAt(i)).norm2();
    }
    return norm2;
  }
  
  public float norm    () { return (float) Math.sqrt(norm2()); }
}

