package  isf.vector;

import   sep.io.*;
import juice.util.JuiceUtil;
import  java.io.*;
import  java.net.*;
import   jam.JamObject;
import   jam.vector.*;

/*
  Don't forget to check for space compatibility -- maybe this could
  be handled in a superclass.
  */

/**
 *  Irregularly Sampled Function
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @see       IsfSpace
 * @author    Matt Schwab
 */

public class Isf implements Vector {
  protected IsfSpace   spc;
  protected Elements elements;

  // constructors
  public Isf() {}

  public Isf(int keys, int elements) {
    this.spc     = new IsfSpace(keys, elements);
    this.elements = new ElementsV(this.spc.spaces); 
  }

  public Isf(IsfSpace spc) { 
    this.spc     = spc;
    this.elements = new ElementsV(spc.spaces);
  }

  public Isf(IsfSpace spc, Elements elements) {
    this.spc     = spc;
    this.elements = elements;
  }

  /** copy constructor */
  protected Isf(Isf src) {
    this.spc     = (IsfSpace)   src.spc.copy();
    this.elements = (Elements) src.elements.copy();
  }

  /**
   * WARNING: the space copied is a shallow copy, meaning
   * that if you subsequently change any of the internals of the
   * space of one vector, then the space of the other vector is
   * affected.
   */
  public JamObject copy() { return new Isf(this); }

  public boolean equals(Object obj) {
    if (  this == obj                 ) return true;
    if (! (obj instanceof Isf )       ) return false;
    if (! spc.equals(((Isf) obj).spc) ) return false; 
    if (! elements.equals(((Isf) obj).elements) ) return false;
    return true;
  }

  // methods from java Vector 
  public int    getNumberOfElements() { return elements.getNumberOfElements(); }

  // methods from jam Vector
  public void neg() { 
    elements.neg(); 
  }
  public void neg(Vector vec2) { 
    elements.neg( ((Isf) vec2).elements ); 
  }
  public void addScale(float a, Vector vec2) { 
    elements.addScale(a, ((Isf) vec2).elements); 
  }
  public void addScale(float a, Vector vec1, Vector vec2) {
    elements.addScale(a, ((Isf) vec1).elements, ((Isf) vec2).elements); 
  }
  public void add(Vector vec2) { 
    elements.add( ((Isf) vec2).elements); 
  }
  public void add(Vector vec1, Vector vec2) { 
    elements.add( ((Isf) vec1).elements, ((Isf)vec2).elements); 
  }
  public void sub(Vector vec2) { 
    elements.sub( ((Isf) vec2).elements); 
  }
  public void sub(Vector vec1, Vector vec2) { 
    elements.sub( ((Isf) vec1).elements, ((Isf)vec2).elements); 
  }
  public void scale(float a) { 
    elements.scale(a); 
  }
  public void scale(float  a, Vector vec2 ) { 
    elements.scale(a, ((Isf) vec2).elements); 
  }

   // Fill
  public void copyFrom(Vector src) { 
    elements.copyFrom(((Isf) src).elements); 
  }
  public void zero() { 
    elements.zero(); 
  } 

  // Inner product and related operations
  public float dot(Vector vec2) { 
    return elements.dot( ((Isf) vec2).elements ); 
  }
  public float norm2   () { return elements.norm2(); } 
  public float norm    () { return elements.norm2(); }
  public Space getSpace() { return spc; }
  
  // I/O

  public String toString() {
    return ("Isf Space :\n" + spc + "\n" +
	    "Isf Vector:\n" + elements + "\n" );
  }
}
