package isf.vector;

import  jam.vector.*;
import  juice.util.JuiceUtil;
import  jam.JamObject;

/**
 * Factory to create or modify <TT>Isf</TT>s 
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * All the physical implementations are hidden 
 * (Spaces, Keys, Elements, CoordMap)
 *
 * @see       Isf
 * @author    Matthias Schwab
 */
public class IsfFactory implements JamObject{
  Spaces  spaces; 
  Keys  keys;
  Elements  elements;
  CoordMap coorMap; 

  // constructors
  /** 
   * builds an Isf with random coordinate values and random RealVec 
   * elements. 
   * 
   * @param keys     is the number of coordinate values attached to 
   *                 each element.
   * @param elements is the number of irregular data points.
   */
  public IsfFactory(int keys, int elements) {
    this.spaces = new SpacesV(juice.vector.RealSpc.getSpace(), elements);
    this.keys = new KeysV(keys);
    this.elements = new ElementsV(spaces);
    this.coorMap = new CoordMapV(keys, elements);
  }
  public IsfFactory(Spaces  spaces, 
		    Keys  keys,
		    Elements  elements,
		    CoordMap coorMap) { 
    this.spaces = spaces; 
    this.keys = keys;
    this.elements = elements;
    this.coorMap = coorMap;
  }
  public IsfFactory(IsfSpace isfSpc, Isf isfVec) {
    this.spaces = isfSpc.spaces; 
    this.keys = isfSpc.keys;
    this.elements = isfVec.elements;
    this.coorMap = isfSpc.coorMap;
  }  

  /* I think we want this: so that a set of Isf vectors can be treated 
   * consistently. Not implemented yet. 
   * public IsfFactory(IsfSpace isfSpc, Isf[] isfVec) 
   */

  /** not implemented yet 
   * public IsfFactory(Rsf rsf) {} //assume 2-D with RealVector elements 
   */

  /** copy constructor */
  protected IsfFactory(IsfFactory src) {
    this.spaces = (Spaces) src.spaces.copy(); 
    this.keys = (Keys) src.keys.copy();
    this.elements = (Elements) src.elements.copy();
    this.coorMap = (CoordMap) src.coorMap.copy();    
  }

  public JamObject copy() { return new IsfFactory(this); }

  // comparitors
  public boolean equals(Object spc ) {
    if (  this == spc                               ) return true ;
    if (! (spc instanceof IsfFactory)               ) return false;
    if (! spaces.equals(((IsfFactory)spc).spaces) ) return false;
    if (! keys.equals(((IsfFactory)spc).keys) ) return false;
    if (! elements.equals(((IsfFactory)spc).elements) ) return false;
    if (! coorMap.equals(((IsfFactory)spc).coorMap) ) return false;
    return true;
  }

  // accessors
  public IsfSpace getIsfSpace() {
    return new IsfSpace(spaces, keys, coorMap); 
  }
  public Isf      getIsf     () { 
    return new Isf( this.getIsfSpace(), elements );
  }

  public Spaces  getElementSpaces () { return spaces; }
  public Keys  getKeys          () { return keys; }
  public Elements  getElementVectors() { return elements; }
  public CoordMap    getCoordinates   () { return coorMap; }
  
  // accessors
  public int getNumberOfElements() { return elements.getNumberOfElements(); }
  public int getNumberOfKeys    () { return keys.getNumberOfKeys    (); }

  // I/O
  public String toString() { 
    StringBuffer buf = new StringBuffer();
    buf.append("Element Spaces  :\n" + spaces + "\n");
    buf.append("Keys :           \n" + keys + "\n");
    buf.append("Element Vectors :\n" + elements + "\n");
    buf.append("Coordinates:     \n" + coorMap + "\n");
    return buf.toString();
  }

  // Vector - Database accessors

  protected void addElement(Vector vec, java.util.Vector loc) {
    spaces.addElement(vec.getSpace()); 
    elements.addElement(vec           ); 
    coorMap.addElement(loc           );
  }
  protected void insertElementAt(Vector vec, java.util.Vector loc, int i) {
    spaces.insertElementAt(vec.getSpace(), i);
    elements.insertElementAt(vec,            i);
    coorMap.insertElementAt(loc,            i);
  }

  protected void removeElementAt(int i) {
    spaces.removeElementAt(i); 
    elements.removeElementAt(i); 
    coorMap.removeElementAt(i); 
  }
}


 




