package isf.vector;

import  jam.vector.*;
import  juice.util.JuiceUtil;
import  jam.JamObject;

/**
 * vector space for irregular data
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * I may need an interface between the Space interface and these 
 * IsfSpace implementations. 
 *
 * @see       Isf 
 * @see       IsfFactory 
 * @see       CoordMapV
 * @author    Matthias Schwab
 */
public class IsfSpace implements Space {
  Spaces  spaces; 
  Keys  keys;
  CoordMap coorMap;

  // constructors
  /** 
   * builds an IsfSpace with zero default values.
   *
   * @param elements is the number of irregular data points.
   * @param keys     is the number of coordinate values that correspond to 
   *                 each data point.
   */
  public IsfSpace(int keys, int elements) {
    this.spaces = new SpacesV (juice.vector.RealSpc.getSpace(), elements); 
    this.keys = new KeysV (keys          );
    this.coorMap = new CoordMapV(keys, elements); 
  } 
  public IsfSpace(Spaces spaces, 
		  Keys keys, 
		  CoordMap coorMap) { 
    this.spaces = spaces; 
    this.keys = keys;
    this.coorMap = coorMap; 
  } 

  /** not implemented yet 
   * public IsfSpace(Rsf rsf) {} //assume 2-D with RealVector elements 
   */

  /** copy constructor */
  protected IsfSpace(IsfSpace src) {
    this.spaces = (Spaces)  src.spaces.copy(); 
    this.keys = (Keys)  src.keys.copy();
    this.coorMap = (CoordMap) src.coorMap.copy(); 
  }

  public JamObject copy() { return new IsfSpace(this); }

  public void rand() { this.coorMap.rand(); }

  // comparitor
  public boolean equals(Object spc) {
    if (  this == spc                             ) return true ;
    if (! (spc instanceof IsfSpace)               ) return false;
    if (! spaces.equals(((IsfSpace)spc).spaces) ) return false;
    if (! keys.equals(((IsfSpace)spc).keys) ) return false;
    if (! coorMap.equals(((IsfSpace)spc).coorMap) ) return false;
    return true;
  }

  // are they really needed?
  Spaces  getSpacesOfElements  () { return spaces           ; }
  Keys  getKeys              () { return keys           ; }
  CoordMap getCoordinates       () { return coorMap           ; }

  public Vector  newMember()         { return new Isf(this)            ; }
  public boolean hasMember(Vector v) { return this.equals(v.getSpace()); }
  
  // accessors
  // I probably want other accessors, such as enumeration, nearestNeighbor,
  // indexOf etc. 
  // I could double check this information with all other variables 
  // maybe i should have a consistency method that checks these things? 

  public int getNumberOfElements() {return coorMap.getNumberOfElements();}
  public int getNumberOfKeys()     {return coorMap.getNumberOfKeys    ();}

  // I/O
  public String toString() { 
    StringBuffer buf = new StringBuffer();
    buf.append("IsfSpace :           \n"                  );
    buf.append("Spaces of Elements : \n" + spaces  + "\n");
    buf.append("Keys :               \n" + keys  + "\n");
    buf.append("Coordinates:         \n" + coorMap  + "\n");
    return buf.toString();
  }
}


