package net.www.protocol.crypt;

import java.io.*;
import java.net.*;

public class Handler extends URLStreamHandler {
  String cryptype;

  protected void parseURL(URL u, String spec, int start, int end) {
    int slash = spec.indexOf('/');
    cryptype  = spec.substring(start,slash);
    start     = slash;
    super.parseURL(u, spec, start, end);
  }

  protected URLConnection openConnection(URL url) throws IOException {
    return new CryptURLConnection(url, cryptype);
  }
}

class CryptURLConnection extends URLConnection {
  CryptInputStream cis;
  static int defaultPort = 80;

  CryptURLConnection(URL url, String cryptype) throws IOException {

    super(url);
    try {
      String name = "net.www.protocol.crypt." + cryptype + "CryptInputStream";
      cis = (CryptInputStream)Class.forName(name).newInstance();
    }
    catch(Exception e){}
  }

  synchronized public void connect() throws IOException {
    int port;
    if(cis == null)
      throw new IOException("Crypt Class not found");
    if((port = url.getPort()) == -1) 
      port = defaultPort;
    Socket s = new Socket (url.getHost(),port);
    
    // Send the filename in plaintext (according to http protocol
    OutputStream server = s.getOutputStream();
    new PrintStream(server).println("GET " + url.getFile());
    
    // Initialize the CryptInputStream
    cis.set(s.getInputStream(), server);
    connected = true;
  }

  synchronized public InputStream getInputStream() throws IOException {

    if(!connected) 
      connect();
    return(cis);
  }

  public String getContentType() {
    return guessContentTypeFromName(url.getFile());
  }
}

