package net.www.protocol.sepdata;

import java.io.*;
import java.net.*;

public class SepDataHandler extends URLStreamHandler {
  String sepDataType;

  protected void parseURL(URL u, String spec, int start, int end) {
    int slash   = spec.indexOf('/');
    sepDataType = spec.substring(start,slash);
    start       = slash;
    super.parseURL(u, spec, start, end);
  }

  protected URLConnection openConnection(URL url) throws IOException {
    return new SepDataURLConnection(url, sepDataType);
  }
}

class SepDataURLConnection extends URLConnection {
  SepDataInputStream cis;
  static int defaultPort = 80;

  SepDataURLConnection(URL url, String sepDataType) throws IOException {

    super(url);
    try {
      String name = 
	"net.www.protocol.sepdata." + sepDataType + "SepDataInputStream";
      cis = (SepDataInputStream)Class.forName(name).newInstance();
    }
    catch(Exception e){}
  }

  synchronized public void connect() throws IOException {
    int port;
    if(cis == null)
      throw new IOException("SepData Class not found");
    if((port = url.getPort()) == -1) 
      port = defaultPort;
    Socket s = new Socket (url.getHost(),port);
    
    // Send the filename in plaintext (according to http protocol
    OutputStream server = s.getOutputStream();
    new PrintStream(server).println("GET " + url.getFile());

    // Initialize the SepDataInputStream
    cis.set(s.getInputStream(), server);
    connected = true;
  }

  synchronized public InputStream getInputStream() throws IOException {

    if(!connected) 
      connect();
    return(cis);
  }

  /**
   * enhance here to return the correct ContentType for SepData
   */
  public String getContentType() {
    //return guessContentTypeFromName(url.getFile());
    return "extension_token/x-SepRegularData";
  }
}

