package rn.operator;

import    jam.JamObject;
import    jam.vector.*;
import    rn.vector.*;

/**
 * <BR><!--#include file="./source.inc"-->
 *
 * @author Matthias Schwab
 */
public class Matrix {
  public float[][] d;
  int row, col;

  /** 
   * fast (second) index is the row dimension.<BR> 
   *      col -->      <BR>
   * r | r11 r12 r13   <BR>
   * o | r21 r22 r23   <BR>
   * w V               <BR>
   *                   <BR>
   * The matrix has 3 columns and 2 rows.
   * The column index (0-2) is the fast, second index 
   * (the one you want to use inside of a for loop). 
   * Consequently, m.v[1][2] = r12 and vice versa.
   */ 
  public Matrix(int row, int col) { 
    this.row = row; 
    this.col = col;
    d = new float[row][col]; 
  }

  /** fast (second) index is the row dimension. */ 
  public     Matrix(float[][] v) {
    this(v.length, v[0].length);
    for (int i1=0; i1 < row; i1++) 
    for (int i0=0; i0 < col; i0++) 
      d[i1][i0] = v[i1][i0]; 
  }
  public     Matrix(Matrix src) { 
    this(src.row,src.col);
    for (int i1=0; i1 < row; i1++) 
    for (int i0=0; i0 < col; i0++) 
      d[i1][i0] = src.d[i1][i0];
  }
  public Matrix copy()         { return new Matrix(this); }

  public void      setElement(int row, int col, float v) { 
    this.d[row][col] = v; 
  }
  public float     getElement(int row, int col)          { 
    return this.d[row][col]; 
  }
  public boolean   equals(Object obj) {
    if (this == obj) return true;
    if (! (         obj instanceof Matrix ))            return false;
    if (this.row != ((Matrix) obj).row)                 return false;
    if (this.col != ((Matrix) obj).col)                 return false;
    for (int i1=0; i1 < row; i1++)
    for (int i0=0; i0 < col; i0++)
      if ( ((Matrix) obj).d[i1][i0] != this.d[i1][i0])  return false;
    return true;
  }

  RnVec getRow(int j) { 
    // float[] tmp = new float[row];
    // for (int i=0; i < col; i++) tmp[i] = d[j][i];
    // return new RnVec(tmp); 
    return new RnVec(d[j]);
  }

  RnVec getCol(int j) {
    float[] tmp = new float[row];
    for (int i=0; i < row; i++) tmp[i] = d[i][j];
    return new RnVec(tmp); 
  }

  public String toString() { 
    StringBuffer buffy = new StringBuffer(); 
    buffy.append("Matrix: \n"); 
    int maxRow = (row > 15) ? 15 : row;
    int maxCol = (col > 15) ? 15 : col;
System.err.println("I am here: " + maxRow + " " + maxCol);
    for   (int i1=0; i1 < maxRow; i1++) {
System.err.println("but here?");
      buffy.append("d[" + i1 + "][1:" + maxCol + "]: ");
      for (int i0=0; i0 < maxCol; i0++) {
	buffy.append(d[i1][i0] + " ");
      }
      buffy.append("\n"); 
    }
    return buffy.toString();
  }
}
