package rn.vector;

import    jam.JamObject;
import    jam.vector.*;
import   java.util.Random;

/**
 * <BR><!--#include file="./source.inc"-->
 *
 * @author Matthias Schwab
 */
public class RnVec implements Vector {
  public float[] d;
  RnSpc  space; 

  public     RnVec(RnSpc s)   { 
    space = new RnSpc(s);
    d = new float[s.getNSamples()];
  }
  public     RnVec(float[] v)   {
    this(new RnSpc(v.length));
    for (int i=0; i < d.length; i++) 
      d[i] = v[i];
  }
  public     RnVec(RnVec src) { 
    this(src.space);
    for (int i=0; i < d.length; i++) 
      d[i] = src.d[i];
  }
  public JamObject copy()         { return new RnVec(this); }

  public void      setDatum(int index, float v) { this.d[index] = v   ; }
  public float     getDatum(int index)          { return this.d[index]; }

  public boolean   equals(Object obj) {
    if (this == obj) return true;
    if (! (        obj instanceof Vector))             return false;
    if (! (        obj instanceof RnVec ))             return false;
    if (! ((RnVec) obj).getSpace().equals(this.space)) return false;
    for (int i=0; i < d.length; i++)
      if ( ((RnVec) obj).d[i] != this.d[i])            return false;
    return true;
  }

  public Space     getSpace() { return space; }

  public void      neg     () {
    for (int i=0; i < d.length; i++) d[i] = - d[i];}
  public void      neg     (Vector vec2) {
    for (int i=0; i < d.length; i++) d[i] = - ((RnVec)vec2).d[i];}
  public void      add     (Vector vec2) {
    for (int i=0; i < d.length; i++) d[i] += ((RnVec)vec2).d[i];}
  public void      add     (Vector vec1, Vector vec2) {
    for (int i=0; i < d.length; i++) 
      d[i] = ((RnVec)vec1).d[i] + ((RnVec)vec2).d[i];}
  public void      addScale(float a, Vector vec2) {
    for (int i=0; i < d.length; i++) d[i] += ((RnVec)vec2).d[i] * a;}
  public void      addScale(float a, Vector vec1, Vector vec2) {
    for (int i=0; i < d.length; i++) 
      d[i] = ((RnVec)vec2).d[i] * a + ((RnVec)vec1).d[i];}
  public void      sub     (Vector vec2) {
    for (int i=0; i < d.length; i++) d[i] -= ((RnVec)vec2).d[i];}
  public void      sub     (Vector vec1, Vector vec2) {
    for (int i=0; i < d.length; i++) 
      d[i] = ((RnVec)vec1).d[i] - ((RnVec)vec2).d[i];}
  public void      scale   (float  a   ) {
    for (int i=0; i < d.length; i++) d[i] *= a;}
  public void      scale   (float  a   , Vector vec2) {
    for (int i=0; i < d.length; i++) d[i] = a * ((RnVec) vec2).d[i];}
  public void      copyFrom(Vector src) {
    if (! ((RnVec) src).getSpace().equals(space)) 
      System.err.println("Error: dimensions incompatible");
    for (int i=0; i < d.length; i++) d[i] = ((RnVec) src).d[i];}
  public void      zero    ()            {
    for (int i=0; i < d.length; i++) d[i] =  0F; }

  public float     dot     (Vector vec2) {
    float tmp = 0.f; 
    for (int i=0; i < d.length; i++) tmp += d[i] * ((RnVec) vec2).d[i]; 
    return tmp; }
  public float     norm2   ()            {
    return this.dot(this); }
  public float     norm    ()            {
    return (float) Math.sqrt(this.norm2()); }

  public void      rand() {
    for (int i = 0; i < d.length; i++ )
      d[i] = 2*((float) Math.random()) - 1.f;
  }
  public void      rand(Random r) {
    for (int i = 0; i < d.length; i++ )
      d[i] = 2*(r.nextFloat()) - 1.f;
  }
  public void      fill(float f) {
    for (int i = 0; i < d.length; i++ )
      d[i] = f;
  }


  public String toString() { 
    StringBuffer buffy = new StringBuffer(); 
    buffy.append("RnVec: \n"); 
    int max = (d.length > 15) ? 15 : d.length; 
    for (int i=0; i < max; i++) {
      buffy.append("d[" + i + "]: " + d[i] + " ");
    }
    return buffy.toString();
  }
}
