package   rsfF.solver;

import    sep.operator.SepSelfAdjointOperator;
import    jam.operator.*;
import    jam.vector.*;
import    rsfF.vector.*;
import    jam.JamObject;

/**
 * This probably doesn't belong in rsfF.solver, 
 * but MissingDataEquations is the only
 * thing that currently uses it.  Also, the name is probably not so good.
 *
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class SepMask extends SepSelfAdjointOperator implements Mask {
  protected boolean[] mask;
  protected boolean   maskZero;
  protected SepMask  complement = null;

  /**
   * Create a mask which zeroes the components which are zero in maskRsf.<BR>
   * @param maskRsf may be subsequently changed without affecting "this".
   */
  public SepMask(Rsf maskRsf) {
    super(maskRsf.getSpace());
    maskZero = false;
    createMask(maskRsf);
  }

  protected SepMask(SepMask src) {
    super(src);
    this.maskZero = src.maskZero;
    int n = src.mask.length;
    this.mask = new boolean[n];
    for (int i = 0; i < n; i++)
      this.mask[i] = src.mask[i];
  }

  void createMask(Rsf rsf) {
    float[] vals = rsf.d;
    int n = vals.length;
    mask = new boolean[n];
    for (int i = 0; i < n; i++)
      mask[i] = (vals[i] == 0F) ? maskZero:(!maskZero);
  }

  void createComplement() {
    try {
      this.complement = (SepMask) this.clone();
    }
    catch (Exception e) { System.err.println(e); }
    this.complement.complement = this;
    this.complement.maskZero = !this.maskZero;
  }

  /**
   * The complement of a <TT>SepMask</TT>, M, is the mask, C, which zeroes exactly
   * those components which M does not zero.<BR>
   * Thus, <TT>C = I - M</TT>, and M.getComplement().getComplement() = M<BR>
   * (Here, <TT>I</TT> is the identity operator).<BR><BR>
   */
  public Mask getComplement() {
    if (complement == null) createComplement();
    return complement;
  }

  protected void apply(boolean add, Vector dom, Vector ran) {
    float[] x = ((Rsf) dom).d;
    float[] y = ((Rsf) ran).d;
    int n = x.length;
    if (add) {
      for (int i = 0; i < x.length; i++)
	if (mask[i] != maskZero)
	  y[i] += x[i];
    }
    else
      for (int i = 0; i < x.length; i++)
	y[i] = (mask[i] == maskZero) ? 0F:x[i];
  }

  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof SepMask)) return false;
    SepMask msk2 = (SepMask) obj2;
    return (super.equals(msk2) &&
	    this.maskZero == msk2.maskZero &&
	    juice.util.JuiceUtil.equalArrays(this.mask, msk2.mask));
  }

  public JamObject  copy() { return new SepMask(this); }

  public String toString() {
    return ("SepMask:\n\tDomain and Range:\n" + space + "\n\tMask:\n" +
	    juice.util.JuiceUtil.arrayToString(mask, " "));
  }
}
