package  rsfF.util;

import   rsf.vector.Axis;
import   rsfF.vector.*;
import    sep.io.*;
import   java.io.*;
import   java.util.*;
/** 
 * This class generates a synthetic Rsf. 
 * <BR><!--#include file="./source.inc"--><BR>
 * This should probably be an Rsf Factory.
 * The setZero(), setConstant(), setRandom(), setSpike()
 * methods vary the data values. 
 * We may want to have more sophisticated methods to define set of spikes.
 * (e.g. iterator as arguments, or hyperplanes like it was possible in the 
 * old Spike).
 * We may want to have an independent Random tool that can create various
 *    random distributions. 
 *
 * @author Matthias Schwab
 */

public class Spike {
  private Rsf rsf;

  public Spike(Rsf rsf) {
    this.rsf = rsf;
  }

  public Spike(InputStream istr) {
    this.rsf = new Rsf(istr); 
  }

  public Spike(Axis[] axes) {
    this.rsf = new Rsf(axes); 
  }
  public Spike(int[] nSamples) {
    this.rsf = new Rsf(nSamples);
  }
  public Spike(int n) {
    this.rsf = new Rsf(n);
  }
  public void setZero() {
    rsf.zero();
  }
  public void setDatum(int[] indices, float magnitude) {
    if (indices.length != rsf.getDimension()) {
      System.err.println("incorrect number of indices!");
    } 
    else {
      rsf.setDatum(indices, magnitude);
    }
  }
  public void setConstant(float magnitude) {
    float[] data = rsf.getData();
    for (int i=0; i < rsf.getTotalNSamples(); i++) {
      data[i] = magnitude;
    }
  }
  public void setRandom(){
    rsf.rand();
  }

  public Rsf getRsf(){
    return this.rsf; 
  }

  public String toString(){
    return this.rsf.toString();
  } 

  public static void main(String[] args) throws java.io.IOException {

    ArgumentKeeper command = new ArgumentKeeper(args);

    // construct the rsf
    // this should be inside of ArgumentKeeper which should share it with
    // ParaKeeper ... 
    int dim = 1;
    while ( command.containsKey("n" + dim) && command.getInt("n" + dim) > 1 ) {
      dim++;
    }
    dim = dim - 1;

    Axis[] ax = new Axis[dim];

    int    nsamples  ;
    float  offset    ;
    float  increment ;
    String label     ;
    String unit      ;

    for(int i=1; i <= dim; i++) { 
      //defaults
      nsamples  = 1  ;
      offset    = 0.F;
      increment = 1.F;
      label     = "Get a label!";
      unit      = "Get a unit! ";
      if (command.containsKey("n"     + i)) {
	nsamples  = command.getInt(  "n"      + i);
      }
      if (command.containsKey("o"     + i)) {
	offset    = command.getFloat("o"      + i);
      }
      if (command.containsKey("d"     + i)) {
	increment = command.getFloat("d"      + i);
      }
      if (command.containsKey("label" + i)) {
	label     = command.getString("label" + i);
      }
      if (command.containsKey("unit"  + i)) {
	unit      = command.getString("unit"  + i);
      }
      ax[i-1] = new Axis( nsamples, increment, offset, label, unit );
    }
    Spike spike = new Spike(ax);

    // Set the spikes 

    int nspikes = 1;
    while ( command.containsKey("k" + nspikes) ) {
      nspikes++;
    }
    nspikes = nspikes - 1;

    int[] coordInt  = new int[dim]; 
    for(int i=1; i <= nspikes; i++) { 
      String coordString = command.getString("k"  + i);
      StringTokenizer tok = new StringTokenizer( coordString, "," );
      if (tok.countTokens() == dim) {
	int k = 0; 
	while (tok.hasMoreTokens()) {
	  coordInt[k] = Integer.valueOf(tok.nextToken()).intValue() -1 ;
	  k++;
	} 
      }
      else {
	System.err.println("I should allow for hypersurfaces as Spike did");
      }
      spike.setDatum(coordInt,1.F);
    }

    // write it out!
    Rsf   myrsf = spike.getRsf();
    myrsf.write(System.out);
  }
}
