package  rsfF.vector;

import   rsf.vector.Axis;
import   rsf.vector.DataAxis;
import   sep.io.*;
import   jam.vector.*;
import juice.util.JuiceUtil;
import  java.io.*;
import  java.net.*;
import  java.util.Random;
import   jam.JamObject;

/*
  Don't forget to check for space compatibility -- maybe this could
  be handled in a superclass.
  */

/**
 *  Regularly Sampled Function
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @see       RsfSpace
 * @author    Joel Schroeder
 */

public class Rsf implements Vector {
  protected String   title = "Unknown";
  public    float[]  d    ; 
  protected RsfSpace space;
  
  // constructors
  public Rsf() {}
  
  public Rsf(InputStream istr) {
    read(istr);
  }
  
  public Rsf(String fileName ) {
    try {
      read(new FileInputStream(new File(fileName)));
    }
    catch(Exception e ) {
      System.err.println(e );
      System.exit(-1);
    }
  }
  
  void read(InputStream istr) {
    RegularData rsfData = null;
    try { rsfData = new RegularData(istr); }
    catch(Exception e) { System.err.println("Exception: " + e ); }
    title             = rsfData.getString("title","Unknown");
    String dataLabel  = rsfData.getString("label","Unknown");
    String dataUnit   = rsfData.getString("unit", "Unknown");
    int    dim        = rsfData.getInt("dim");
    DataAxis dax      = new DataAxis(dataLabel, dataUnit); 
    Axis[] ax         = new Axis[dim];
    for (int i = 1; i <= dim; i++ ) {
      int    nsamples = rsfData.getInt   ("n"     + i);
      float  delta    = rsfData.getFloat ("d"     + i);
      float  offset   = rsfData.getFloat ("o"     + i);
      String label    = rsfData.getString("label" + i);
      String unit     = rsfData.getString("unit"  + i);
      ax[i-1] = new Axis(nsamples, delta, offset, label, unit );
    }
    space = new RsfSpace(ax, dax);
    d = (float[])rsfData.getData(); 
    if (space.getTotalNSamples() != d.length )
      System.out.println("Error: Rsf data stream size different from " +
			 "header's specification." );
  }
  
  public Rsf(Axis[]   axes, DataAxis dataAxis) {
    space = new RsfSpace(axes, dataAxis);
    this.allocateData();
  }
  public Rsf(Axis[]   axes     ) {
    space = new RsfSpace(axes );
    this.allocateData();
  }
  public Rsf(int []   nSamples ) {
    space = new RsfSpace(nSamples);
    this.allocateData();
  }
  public Rsf(int      n        ) {
    space = new RsfSpace(n);
    this.allocateData();
  }
  public Rsf(int      rows, int cols) {
    space = new RsfSpace(rows, cols);
    this.allocateData();
  }
  public Rsf(RsfSpace space    ) {
    this.space = space;
    this.allocateData();
  }

  /** copy constructor */
  protected Rsf(Rsf src) {
    this.space = src.space;
    this.d = new float[src.d.length];
    for (int i = 0; i < d.length; i++ )
      d[i] = src.d[i];
  }
  
  public void   setTitle(String title) { this.title = title; }
  public String getTitle(            ) { return this.title;  }

  /**
   * WARNING: the space copied is a shallow copy, meaning
   * that if you subsequently change any of the internals of the
   * space of one vector, then the space of the other vector is
   * affected.
   */
  public JamObject copy() { return new Rsf(this); }

  public boolean equals(Object rsf2) {
    if (this == rsf2) return true;
    if (!(rsf2 instanceof Rsf )) return false;
    else return (space.equals(((Rsf) rsf2).space)
		 && JuiceUtil.equalArrays(d, ((Rsf) rsf2).d ));
  }
  
  protected void allocateData() {
    d = new float[((RsfSpace) this.getSpace()).getTotalNSamples()];
  }
  
  public float[] getData() { return d;}
  public void    setData(float[] data) {
    if (data.length != getTotalNSamples()) {
      System.err.println("Error in Rsf.setData(): size of new data is " +
			 "incompatible with the RsfSpace.");
      return;
    }
    d = data;
  }
  /**
   * get/set Datum is for encapsulated access (alternative to direct
   * access to x.d)
   */
  public float 	 getDatum(int[] i) { return d[getIndex(i)]; }
  public float 	 getDatum(int   i) { return d[i]          ; }
  public float 	 getDatum(int   i, int j) {
    return d[getIndex(i,j)];
  }
  
  public void  	 setDatum(int[] i, float v) { d[getIndex(i)] = v; }
  public void  	 setDatum(int   i, float v) { d[i]           = v; }
  public void  	 setDatum(int   i, int j, float v) {
    d[getIndex(i,j)] = v;
  }
  
  public int     getDimension()     { return space.getDimension(); }
  public int[]   getNSamples()      { return space.getNSamples(); }
  public int     getTotalNSamples() { return space.getTotalNSamples(); }
  
  public int     i(int[] i           ) { return space.getIndex(  i); }
  public int     i(int   i           ) { return space.getIndex(  i); }
  public int     i(int i,int j       ) { return space.getIndex(i,j); }
  // This is Wrong
  //  public int     i(int i,int j, int k) { return space.getIndex(i,j); }
  
  /**
   * getIndex needs to be removed (?): replaced by index function i
   * (or is a more explicit "getIndex" naming convention better?)
   */
  public int     getIndex(int[] i    ) { return space.getIndex(  i); }
  public int     getIndex(int i,int j) { return space.getIndex(i,j); }
  public int[]   getIndices(   int i) { return space.getIndices(i); }
  
  // methods from Vector
  public void neg() {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      d[i] = - d[i];
  }
  
  public void neg(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] = - ((Rsf) vec2).d[i];
  }
  
  public void addScale(float a,
		       Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] +=  a * ((Rsf) vec2).d[i];
  }
  
  public void addScale(float a,
		       Vector vec1,
		       Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] = ((Rsf) vec1).d[i] + a * ((Rsf) vec2).d[i];
  }
  
  public void add(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] +=  ((Rsf) vec2).d[i];
  }
  
  public void add(Vector vec1,
		  Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] = ((Rsf) vec1).d[i] + ((Rsf) vec2).d[i];
  }
  public void sub(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] -=  ((Rsf) vec2).d[i];
  }
  
  public void sub(Vector vec1,
		  Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] = ((Rsf) vec1).d[i] - ((Rsf) vec2).d[i];
  }
  
  public void scale(float        a    ) {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] *= a;
  }
  
  public void scale(float        a    ,
		    Vector vec ) {
    // throw if this and vec spaces are unequal
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i]  = a * ((Rsf) vec).d[i];
  }
  
  /** Point-wise multiplication */
  /*
    public Vec times(Vec   vec2 	) {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ ) {
    this.d[i] *= ((Rsf) vec2).d[i]; }
    return this;
  }
    */  
  /** Point-wise multiplication */
  /*
    public Vec times(Vec   vec1 , Vec   vec2 ) {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ ) {
    this.d[i] = ((Rsf) vec1).d[i] * ((Rsf) vec2).d[i];
}
    return this;
}
    */
  /** Point-wise multiplication plus addition (like addScale) */
  /*
    public Vec addTimes(Vec vec2 ) {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ ) {
    this.d[i] += this.d[i] * ((Rsf) vec2).d[i];
}
    return this;
}
    */
  /** Point-wise multiplication plus addition (like addScale) */
  /*
    public Vec addTimes(Vec vec1, Vec vec2 ) {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ ) {
    this.d[i] += ((Rsf) vec1).d[i] * ((Rsf) vec2).d[i];
}
    return this;
}
    */
  
  // Fill
  public void copyFrom(Vector src) {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] = ((Rsf) src).d[i];
  }

  public void zero() { fill(0); }
  public void fill(float a) {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] = a;
  }
  public void rand() {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] = 2*((float) Math.random()) - 1.f;
  }
  public void rand(Random r) {
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ )
      this.d[i] = 2*(r.nextFloat()) - 1.f;
  }
  
  // Inner product and related operations
  public float dot  (Vector vec2) {
    float innr = 0f;
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ ) {
      innr += this.d[i] * ((Rsf) vec2).d[i];
}
    return innr;
}
  
  public float norm2() {
    float norm2 = 0;
    for (int i = 0; i < ((RsfSpace) this.getSpace()).getTotalNSamples(); i++ ) {
      norm2 += this.d[i] * this.d[i];
}
    return norm2;
}
  
  public float norm    () { return (float) Math.sqrt(norm2()); }
  
  public Space getSpace() { return space; }
  
  // I/O
  public String toString() {
    return ("Rsf: "          + title       + "\n" +
	    "Space:"         + getSpace()  + "\n" +
	    "Sample data:"                 + "\n" +
	    JuiceUtil.arrayToString(d, " ")	   );
  }

  public void write  (OutputStream outstr ) {
    int dim = space.getDimension();
    ParaKeeper pars = new ParaKeeper();
    pars.set("in" , "\"stdin\"");
    pars.set("dim", dim);
    pars.set("title", this.title); 
    pars.set("label", space.getDataAxis().label); 
    pars.set("unit" , space.getDataAxis().unit );
    for (int i = 1; i <= dim; i++ ) {
      pars.set("o"     + i, space.getAxes()[i-1].offset   );
      pars.set("d"     + i, space.getAxes()[i-1].delta    );
      pars.set("n"     + i, space.getAxes()[i-1].nsamples );
      pars.set("label" + i, space.getAxes()[i-1].label    );
      pars.set("unit"  + i, space.getAxes()[i-1].unit     );
    }
    try {
      new RegularData(d, pars).write(outstr);
    } catch(Exception e ) { System.err.println("Exception:" + e ); }
  }
  
  public void write  (String fileName  ) {
    try {
      this.write(new FileOutputStream(new File(fileName)));
    } catch(Exception e) { System.err.println(e); }
  }

}
