package rsfF.vector;

import  rsf.vector.Axis;
import  rsf.vector.DataAxis;
import  jam.vector.*;
import  juice.util.JuiceUtil;

/**
 * vector space containing <TT>Rsf</TT>s
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @see       Rsf
 * @author    Joel Schroeder
 */
public class RsfSpace implements SpaceBasis {
  DataAxis dataAxis; 
  Axis[] axes;  
  
  // constructors
  public RsfSpace() {}

  /** copy constructor */
  protected RsfSpace(RsfSpace src) {
    this.dataAxis = (DataAxis) src.dataAxis.copy();
    int n = src.axes.length;
    this.axes = new Axis[n];
    for (int i = 0; i < n; i++)
      this.axes[i] = (Axis) src.axes[i].copy();
  }

  public RsfSpace(int[] nSamples ) {
    this.dataAxis = new DataAxis();
    this.axes     = new     Axis[nSamples.length];
    for (int i = 0; i < nSamples.length; i++ )
      this.axes[i] = new Axis(nSamples[i]);
  }
  public RsfSpace(int    n) {
    this.dataAxis = new DataAxis();
    this.axes     = new     Axis[1];
    this.axes[0]  = new     Axis(n); }
  public RsfSpace(int    rows, int cols) {
    this.dataAxis = new DataAxis();
    this.axes     = new     Axis[2];
    this.axes[0]  = new     Axis(rows);
    this.axes[1]  = new     Axis(cols); }
  public RsfSpace(Axis[] axes ) {
    this.dataAxis = new DataAxis();
    this.axes = axes; 
  }
  public RsfSpace(Axis[] axes, DataAxis axis) {
    this.dataAxis = axis;
    this.axes     = axes; 
  }

  public int   getIndex  (int[] i) {
    int ind = 0;
    for (int dim = Math.min(axes.length, i.length ) - 1; dim > 0; dim-- )
      ind = (i[dim] + ind) * axes[dim-1].nsamples;
    return i[0] + ind; }
  
  public int   getIndex  (int i) {
    return i ; }
  public int   getIndex  (int i, int j) {
    return i + j * axes[0].nsamples; }
  
  public int[] getIndices(int   i) {
    int[] ind = new int[axes.length];
    getIndices(i, ind);
    return ind;
  }
  
  public void getIndices(int   i, int ind[]) {
    int stride = getStride(ind.length - 1);
    for (int dim = ind.length - 1; dim > 0; dim-- ) {
      ind[dim] = i / stride;
      i -= stride * ind[dim];
      stride /= getNSamples(dim - 1);
    }
    ind[0] = i;
  }

  public jam.JamObject copy() { return new RsfSpace(this); }
  
  // comparitors
  public boolean equals(Object spc2 ) {
    if (this == spc2)                                            return true;
    if (!(spc2 instanceof RsfSpace))                             return false; 
    if (!(JuiceUtil.equalArrays(axes, ((RsfSpace) spc2).axes ))) return false;
    if (!(dataAxis.equals(((RsfSpace) spc2).dataAxis)))          return false;
    return true;
  }
  
  public DataAxis getDataAxis() { return dataAxis; }
  public Axis[]   getAxes    () { return axes    ; }
  
  public Vector  newMember()         { return new Rsf(this); }
  public boolean hasMember(Vector v) { return this.equals(v.getSpace()); }
  public OrthonormalBasis getBasis() { return new RsfBasis(this); }
  
  /**
   * Return a new RsfSpace which is the Cartesian product of two RsfSpaces.<BR>
   * This is similar to creating a <tt>ProductSpace</tt>, but the result of
   * <tt>cross</tt> is a single RsfSpace instead of a <tt>ProductSpace</tt>.
   * 
   * @see jam.vector.ProductSpace
   */
  public RsfSpace cross(RsfSpace spc2) {
    if ( ! this.dataAxis.equals(spc2.dataAxis)) {
      System.err.println("Error in RsfSpace cross: this.dataAxis" + 
			 " and spc2.dataAxis are not equal");
    }; 
    int d1 = this.getDimension();
    int d2 = spc2.getDimension();
    Axis[] newAxes = new Axis[ d1 + d2 ];
    for (int i = 0; i < d1; i++ ) { newAxes[i]      = this.getAxes()[i]; }
    for (int i = 0; i < d2; i++ ) { newAxes[i + d1] = spc2.getAxes()[i]; }
    return new RsfSpace(newAxes); }
  
  // accessors
  public int   getDimension()       { return axes.length; }
  public int[] getDimensions()      {
    int[] dims = new int[axes.length];
    for (int i = 0; i < dims.length; i++)
      dims[i] = axes[i].nsamples;
    return dims;
  }
  public int   getNSamples (int i) {
    return (i < getDimension()) ? (axes[i].nsamples):1; }
  public int   getNSamples (int lo, int hi) {
    int total = 1;
    while (hi >= lo) total *= getNSamples(hi--);
    return total; }
  public int[] getNSamples() {
    int[] dims = new int[axes.length];
    for (int i = 0; i < dims.length; i++)
      dims[i] = axes[i].nsamples;
    return dims;
  }

  /**
   * @return the stride in the storage array corresponding to the specified dimension
   */
  public int   getStride   (int i) { return getNSamples(0, i-1); }
  public int   getTotalNSamples()   { return getNSamples(0,
							 getDimension() - 1); }
  
  // I/O
  public String toString() {
    return ("DataAxis:\n" + dataAxis + "\n" +
	    "Axes:    \n" + JuiceUtil.arrayToString("axis", axes, "\n" ) ); }
  
  /*
    public void write  (PrintStream pstr ) {
    pstr.println(this ); }
    */
}

class RsfBasis implements OrthonormalBasis {
  RsfSpace space;

  public RsfBasis(RsfSpace space) { this.space = space; }

  public SpaceBasis getSpace() { return space; }
  public int        size()     { return space.getTotalNSamples(); }
  public Vector     getVector(int i) {
    // We rely on the fact? that arrays of floats are initialized to 0
    Rsf r = new Rsf(space);
    r.setDatum(i, 1F);
    return r;
  }

  public jam.JamObject  copy  ()   { return new RsfBasis(space); }

  public boolean    equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof RsfBasis)) return false;
    return (this.space == ((RsfBasis) obj2).space);
  }

  public String toString() {
    StringBuffer sb = new StringBuffer("RsfBasis:\n" +
				       "\tSpace:\n"  + space +
				       "\tBasis:");
    for (int i = 0; i < size(); i++)
      sb.append("\n\t\tVector " + i + ":\n" + getVector(i));
    return sb.toString();
  }

}
